/**************************************************************************
**       Title: load/save policy for SVMApplication for NetCDF files
**    $RCSfile: LoadSaveNetCDF.hh,v $
**   $Revision: 1.5 $$Name:  $
**       $Date: 2005/02/16 09:47:08 $
**   Copyright: GPL $Author: ronneber $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: LoadSaveNetCDF.hh,v $
**  Revision 1.5  2005/02/16 09:47:08  ronneber
**  - corrected include and install paths. Now install directory is
**    .../include/libsvmtl/...
**
**  Revision 1.4  2004/10/26 11:37:20  ronneber
**  moved StDataNetCDF to policy class
**
**  Revision 1.3  2004/09/08 14:42:43  ronneber
**  - adapted to new ParamInfo class
**
**  Revision 1.2  2004/08/27 09:53:55  ronneber
**  - fixed bug in saving crossval results
**
**  Revision 1.1  2004/08/26 08:36:59  ronneber
**  initital import
**
**
**
**************************************************************************/

#ifndef LOADSAVENETCDF_HH
#define LOADSAVENETCDF_HH

#include <iostream>
#include <fstream>
#include <cerrno>

#include "StDataNetCDF.hh"
#include <libsvmtl/StDataCmdLine.hh>
#include <libsvmtl/PrettyOptionPrinter.hh>
#include <libsvmtl/BasicSVMAdapter.hh>
#include <libsvmtl/BasicSVMFactory.hh>
#include <libsvmtl/ParamInfo.hh>

namespace svt
{
  template<typename STDATATYPE>
  class LoadSaveNetCDFTempl
  {
  public:

     ~LoadSaveNetCDFTempl()
          {}
    
    typedef STDATATYPE STDATA;
    

    /*======================================================================*/
    /*! 
     *   get information about the parameters, that are used in
     *   loadFeatureVectors(). The Infos are appended
     *   to the passed array
     *
     *   \param p  array of ParamInfos. The Infos will be appended to
     *             this array
     */
    /*======================================================================*/
    static void getParamInfosForLoadFeatureVectors( std::vector<ParamInfo>& p);

    /*======================================================================*/
    /*! 
     *   Just accesses all parameters, that are used in
     *   loadFeatureVectors() so that they are no longer unknwon to
     *   StDataASCIIFile 
     *
     *   \param cmdline command line
     */
    /*======================================================================*/
    void checkParamsForLoadFeatureVectors(StDataCmdLine& cmdline) const;
    
    
    /*======================================================================*/
    /*! 
     *   load feature vectors
     *
     *   \param cmdline         command line parameters
     *   \param featureVectors  the loaded feature vectors
     *
     *   \exception svt::LoadError file could not be opened 
     */
    /*======================================================================*/
    template< typename FV>
    void loadFeatureVectors( StDataCmdLine& cmdline,  
                              std::vector<FV>& featureVectors);
    
    

    /*======================================================================*/
    /*! 
     *   get information about the parameters, that are used in
     *   saveModel(). The Infos are appended
     *   to the passed array
     *
     *   \param p  array of ParamInfos. The Infos will be appended to
     *             this array
     */
    /*======================================================================*/
    static void getParamInfosForSaveModel( std::vector<ParamInfo>& p);
    
    /*======================================================================*/
    /*! 
     *   Just accesses all parameters, that are used in
     *   saveModel() so that they are no longer unknwon to
     *   StDataASCIIFile 
     *
     *   \param cmdline command line
     */
    /*======================================================================*/
    void checkParamsForSaveModel(StDataCmdLine& cmdline) const;
   

    /*======================================================================*/
    /*! 
     *   save Model. 
     *
     *   \param cmdline         command line parameters
     *   \param svm             the trained SVM
     *   \param detailLevel     wether to save additional training
     *                          infos to model. 0: nothing, 1: only
     *                          statistics, 2: training infos from all
     *                          two-class trainings   
     *
     *   \exception svt::SaveError output file could not be created
     */
    /*======================================================================*/
    template< typename FV>
    void saveModel( StDataCmdLine& cmdline, 
                    BasicSVMAdapter<FV,STDATA>* svm,
                    int detailLevel,
                    std::ostream& os);


    /*======================================================================*/
    /*! 
     *   get information about the parameters, that are used in
     *   createSVMAndLoadModel(). The Infos are appended
     *   to the passed array
     *
     *   \param p  array of ParamInfos. The Infos will be appended to
     *             this array
     */
    /*======================================================================*/
    static void getParamInfosForCreateSVMAndLoadModel( 
        std::vector<ParamInfo>& p);
    
    /*======================================================================*/
    /*! 
     *   create SVM from file and load Model data
     *
     *   \param cmdline         command line parameters
     *   \param modelFileName   file name of modelfile
     *   \param svm             (output) 
     *
     *   \exception svt::LoadError file could not be opened
     *
     *   \return created SVM with loaded Model. The ownership of this object
     *                          goes to caller. So you are responsible
     *                          to delete it after usage 
     */
    /*======================================================================*/
    template< typename FV, typename ALGORITHMS>
    BasicSVMAdapter<FV,STDATA>* createSVMAndLoadModel( 
        StDataCmdLine& cmdline);



    /*======================================================================*/
    /*! 
     *   get information about the parameters, that are used in
     *   createSVMAndLoadModel(). The Infos are appended
     *   to the passed array
     *
     *   \param p  array of ParamInfos. The Infos will be appended to
     *             this array
     */
    /*======================================================================*/
    static void getParamInfosForSaveClassificationResults( 
        std::vector<ParamInfo>& p);


    /*======================================================================*/
    /*! 
     *   Just accesses all parameters, that are used in
     *   saveClassificationResults() so that they are no longer unknwon to
     *   StDataASCIIFile 
     *
     *   \param cmdline command line
     */
    /*======================================================================*/
    bool checkParamsForSaveClassificationResults(
        StDataCmdLine& cmdline) const;

    
    /*======================================================================*/
    /*! 
     *   save classification results. 
     *
     *   \param cmdline  the command line
     *   \param results  predicted labels for each feature vector
     *   \param details  additional classification details in plaint
     *                   text for each feature vector
     *   \param saveDetails wether to save th additional details in
     *                   details vector
     *   \param os       output stream (usually std::cout) for
     *                   additionally infos, like "saving ...", etc.
     *
     */
    /*======================================================================*/
    void saveClassificationResults( 
        StDataCmdLine& cmdline,
        const std::vector<double>& results,
        const std::vector<std::string>& details,
        bool saveDetails,
        std::ostream& os);
  };

  typedef LoadSaveNetCDFTempl<StDataNetCDF> LoadSaveNetCDF;

}

#include "LoadSaveNetCDF.icc"

#endif
