/**************************************************************************
**       Title: structured data class to access NetCDF files
**    $RCSfile: StDataNetCDF.hh,v $
**   $Revision: 1.9 $$Name:  $
**       $Date: 2005/06/21 07:04:53 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: StDataNetCDF.hh,v $
**  Revision 1.9  2005/06/21 07:04:53  fehr
**  64-bit bugfix
**
**  Revision 1.8  2005/04/09 09:48:42  fehr
**  more 64-bit bugfixing
**
**  Revision 1.7  2005/04/09 09:20:39  fehr
**  more 64-bit bugfixing
**
**  Revision 1.6  2005/04/09 08:27:54  fehr
**  netcvs 64-bit long bugfix
**
**  Revision 1.5  2005/02/24 12:57:50  fehr
**  some more 64-bit netcdf bugfixing
**
**  Revision 1.4  2005/02/16 09:47:24  ronneber
**  - corrected include and install paths. Now install directory is
**    .../include/libsvmtl/...
**
**  Revision 1.3  2004/10/26 11:36:56  ronneber
**  moved NcFile to Policy class
**
**  Revision 1.2  2004/09/13 10:13:52  ronneber
**  - documentation update
**
**  Revision 1.1  2004/08/26 08:36:59  ronneber
**  initital import
**
**
**
**************************************************************************/

#ifndef STDATANETCDF
#define STDATANETCDF

#include <cstring>              // Netcdf error handling
#include <cerrno>
#include <string>

#include <netcdf.hh>

#include <libsvmtl/SVMError.hh>

namespace svt
{

  /*-----------------------------------------------------------------------
   *  conversion of c-data-types to NetCDF data types
   *-----------------------------------------------------------------------*/
  template< typename DATATYPE>
  struct MyNetCDFTypeTraits
  {
    typedef void value_type;
    NcType ncType() { return ncNoType; }
  };
  
  // native types
  template<> struct MyNetCDFTypeTraits<unsigned char>
  {
    typedef unsigned char value_type;
    NcType ncType() { return ncByte; }
  };
  
  template<> struct MyNetCDFTypeTraits<char>
  {
    typedef char value_type;
    NcType ncType() { return ncChar; }
  };
  
  template<> struct MyNetCDFTypeTraits<short>
  {
    typedef short value_type;
    NcType ncType() { return ncShort; }
  };
  
  template<> struct MyNetCDFTypeTraits<int>
  {
    typedef int value_type;
    NcType ncType() { return ncInt; }
  };

  template<> struct MyNetCDFTypeTraits<long>
  {
    typedef int value_type;
    NcType ncType() { return ncInt; }
  };

  template<> struct MyNetCDFTypeTraits<float>
  {
    typedef float value_type;
    NcType ncType() { return ncFloat; }
  };
  
  template<> struct MyNetCDFTypeTraits<double>
  {
    typedef double value_type;
    NcType ncType() { return ncDouble; }
  };
  
  // mappings of non-native netCDF types
  template<> struct MyNetCDFTypeTraits< unsigned int>
  {
    typedef int value_type;
    NcType ncType() { return ncInt; }
  };
  
  template<> struct MyNetCDFTypeTraits< bool>
  {
    typedef unsigned char value_type;
    NcType ncType() { return ncByte; }
  };
  


  /*======================================================================*/
  /*!
   *  \class StDataNetCDF
   *  \ingroup stdata
   *  \brief The StDataNetCDF class provies the "structured data"
   *         interface to NetCDF files 
   *
   */
  /*======================================================================*/
  template<typename NcFilePolicy>
  class StDataNetCDFTempl : public NcFilePolicy
  {
  public:
    
    /*======================================================================*/
    /*! 
     *   constructor. Tries to open the specified NetCDF and throws an
     *   error, if it could not be opened
     *
     *   \param path filename of NetCDF file
     *   \param fm   open mode (StDataNetCDF::ReadOnly,
     *               StDataNetCDF::Write, StDataNetCDF::Replace,
     *               StDataNetCDF::New) see NetCDF documentaion for details
     *
     *   \exception LoadError can not open file
     *
     */
    /*======================================================================*/
    StDataNetCDFTempl( const char * path, 
                  typename NcFilePolicy::FileMode fm = ReadOnly);
    
    
    /*======================================================================*/
    /*! 
     *   this constructor is only for compilation of some template
     *   classes. It just throws a runtime error. Never Use it!
     */
    /*======================================================================*/
    StDataNetCDFTempl()
            :NcFile("")
          {
            svt::SVMAssertionFailed err;      
            err << "runtime error: you must not use StDataNetCDF() "
                "constructor without arguments!";
            throw err;
            
          }
    

    /*======================================================================*/
    /*! 
     *   Specify, if getValue() should throw expcetions for unknown keys
     *
     *   \param f true: getValue() should throw expcetions for unknown keys
     *
     */
    /*======================================================================*/
    void setExceptionFlag( bool f)
          {
            _exceptionFlag = f;
          }
    

    /*======================================================================*/
    /*! 
     *   \return value of exceptionFlag (see setExceptionFlag())
     */
    /*======================================================================*/
    bool exceptionFlag() const
          {
            return _exceptionFlag;
          }

    /*======================================================================*/
    /*! 
     *   check wether a value exists for requested key
     *
     *   \param key the key
     *
     *   \return true or false
     */
    /*======================================================================*/
    bool valueExists(  const std::string& key) const
          {
            /*-------------------------------------------------------------
             *  Set NcError to nonfatal and try to get the attribute
             *-------------------------------------------------------------*/
            NcError ncerr( NcError::silent_nonfatal);
            NcAtt *att;
            
            att = this->get_att( key.c_str());
            
            return (ncerr.get_err() == NC_NOERR);
            
          }
    

    /*======================================================================*/
    /*! 
     *   setValue writes/adds a value (with arbitrary type) to the
     *   netCDF File as attribute. 
     *
     *   \param key     the key for that value
     *   \param value   the value itself
     *
     */
    /*======================================================================*/
    template<typename T>
    void setValue( const std::string& key, const T& value);

    void setValue( const std::string& key, const std::string& value);
    void setValue( const std::string& key, const unsigned int& value);
    void setValue( const std::string& key, const unsigned long& value);

    /*======================================================================*/
    /*! 
     *   setArray writes/adds an array of values (with arbitrary type)
     *   to the netCDF File as attribute. 
     *
     *   \param key      the key for that value
     *   \param arrBegin begin of container
     *   \param size     number of elements
     *
     */
    /*======================================================================*/
    template<typename ForwardIter>
    void setArray( const std::string& key, 
                   const ForwardIter& arrBegin,
                   size_t size);
    
    /*======================================================================*/
    /*! 
     *   setFVPList writes/adds a list (or a 1D array) of feature vectors 
     *   to the NetCDF file as 2D Variable. arrBegin must point to
     *   the begin of linear array with _pointers_ to the feature
     *   vectors, e.g. std::vector<BasicFV*>  
     *
     *   \param key      the key for that value
     *   \param arrBegin begin of container
     *   \param size     number of elements in the container
     *
     */
    /*======================================================================*/
    template<typename ForwardIter>
    void setFVArray( const std::string& key, 
                     const ForwardIter& arrBegin,
                     unsigned int size);

    /*======================================================================*/
    /*! 
     *   get a value (with arbitrary type) from the NetCDF attribute
     *   specified by the given key. If key is not  
     *   found, the value remains unchanged and depending on
     *   exceptionFlag() a KeyNotFoundError exception is thrown.
     *
     *   \param key     the key for the requested value
     *   \param value   (output) returned value.  
     *
     *   \exception KeyNotFoundError the requested key does not exist
     *   \exception InvalidDataTypeError the value couldn't be
     *              converted to the requested type
     */
    /*======================================================================*/
    template<typename T> 
    void getValue( const std::string& key, T& value);

    void getValue( const std::string& key, std::string& value);
    

    /*======================================================================*/
    /*! 
     *   get size of Array associated with given key.
     *
     *   \param key      the key for that value
     *
     *   \return  number of elements
     */
    /*======================================================================*/
    unsigned int getArraySize( std::string key);

    /*======================================================================*/
    /*! 
     *   get number of Feature vectors in the list associated with
     *   that key
     *
     *   \param key      the key for that value
     *
     *   \return  number of elements
     *
     */
    /*======================================================================*/
    unsigned int getFVArraySize( std::string key); 


    /*======================================================================*/
    /*! 
     *   getArray. The values must be separated
     *   by separatorChar(), e.g. mymap["nr_sv"] = "150 120". See
     *   getValue() above for further details.
     *
     *   \param key     the key for the requested value
     *   \param arrBegin iterator pointing to your container, that has
     *                  enough space to store all values. You can
     *                  query the required size with
     *                  getArraySize(). 
     *   \param containerSize optional parameter to tell the size of
     *                  your container, so that an error can be
     *                  thrown, wenn it would be exeeded.
     *
     *   \exception KeyNotFoundError the requested key does not exist
     *   \exception InvalidDataTypeError the value couldn't be
     *              converted to the requested type
     */
    /*======================================================================*/
    template<typename ForwardIter> 
    void getArray( const std::string& key, const ForwardIter& arrBegin, 
                   int containerSize=-1);
    
    /*======================================================================*/
    /*! 
     *   get a list (or a 1D array) of feature vectors. The Iterator
     *   must point to a container with Feature vector _pointers_. The
     *   pointers must point to already allocated feature vectors
     *
     *   \param key     the key for the requested value
     *   \param arrBegin iterator pointing to your container, that has
     *                  enough space to store all values. You can
     *                  query the required size with
     *                  getFVArraySize(). 
     *   \param containerSize optional parameter to tell the size of
     *                  your container, so that an error can be
     *                  thrown, wenn it would be exeeded.
     *
     *   \exception KeyNotFoundError the requested key does not exist
     *   \exception InvalidDataTypeError the value string couldn't be
     *              converted to the requested type
     */
    /*======================================================================*/
    template<typename ForwardIter> 
    void getFVArray( const std::string& key, const ForwardIter& arrBegin, 
                    int containerSize=-1);

  protected:

    void getValue( NcAtt* att, bool& value, long index)
          {
            value = (bool)att->as_char( index);
          }

    void getValue( NcAtt* att, char& value, long index)
          {
            value = att->as_char( index);
          }

    void getValue( NcAtt* att, short& value, long index)
          {
            value = att->as_short( index);
          }

    void getValue( NcAtt* att, int& value, long index)
          {
            value = att->as_int( index);
          }
    
    void getValue( NcAtt* att, unsigned int& value, long index)
          {
            value = (unsigned int)att->as_long( index);
          }

    void getValue( NcAtt* att, float& value, long index)
          {
            value = att->as_float( index);
          }

    void getValue( NcAtt* att, double& value, long index)
          {
            value = att->as_double( index);
          }


    /*======================================================================*/
    /*!  
     *   Get NcVar class of a variable, specified by name. Very similiar
     *   to NetCDF's get_var, with the difference, that exceptions are
     *   thrown, when the variable does not exist.
     *
     *   \param      varName  name of the variable
     *
     *   \exception KeyNotFoundError
     *
     *======================================================================*/
    NcVar* xget_var( const std::string &varName) const;
    NcVar* xget_var( int variableIndex) const;
    
    /*======================================================================*/
    /*! 
     *   Get NcAtt class of an attribute, specified by variable and
     *   attribute name. Very similiar to NetCDF's get_att, with the
     *   difference, that exceptions are thrown, when the attribute or the
     *   variable does not exist.
     *
     *   \param      attName  name of the (variable and) attribute,
     *                        e.g. "history" or "fluor09:objective"
     *
     *   \param      varName  name of the variable 
     *                        or "" for global variables
     *
     *   \exception KeyNotFoundError
     *
     *======================================================================*/
    NcAtt* xget_att( std::string attName, std::string varName = "") const;
    

     
  private:
    bool _exceptionFlag;
    std::string _fileName;
    std::string _varName;
    
  };

  typedef StDataNetCDFTempl<NcFile> StDataNetCDF;
  
  
}

#include "StDataNetCDF.icc"
#endif
